@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
-# Copyright (c) 2009  Jendrik Seipp
+# Copyright (c) 2008-2024 Jendrik Seipp
 #
 # RedNotebook is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,54 +22,62 @@ import re
 
 TEXT_RESULT_LENGTH = 42
 
-ALPHA = r'[^\W\d_]'
-ALPHA_NUMERIC = r'\w'
-HEX = r'[0-9A-F]{6}'
-HASHTAG_EXCLUDES = r'%(HEX)s|include' % locals()
-HASHTAG_TEXT = r'%(ALPHA_NUMERIC)s*%(ALPHA)s+%(ALPHA_NUMERIC)s*' % locals()
+HEX_COLOR = r"[0-9a-fA-F]{6}\b"
+CPP_DIRECTIVES = "|".join(
+    f"{keyword}\\b" for keyword in ("include", "define", "ifdef", "ifndef", "endif")
+)
+HASHTAG_EXCLUDES = "|".join((HEX_COLOR, CPP_DIRECTIVES))
+
+ALPHA = r"[^\W\d_]"
+ALPHA_NUMERIC = r"\w"
+HASHTAG_TEXT = r"%(ALPHA_NUMERIC)s*%(ALPHA)s+%(ALPHA_NUMERIC)s*" % locals()
 HASHTAG_PATTERN = (
-    r'(^|[^%(ALPHA_NUMERIC)s&#])(#|\uFF03)(?!%(HASHTAG_EXCLUDES)s)'
-    '(%(HASHTAG_TEXT)s)' % locals())
-HASHTAG = re.compile(HASHTAG_PATTERN, flags=re.I)
+    r"(^|[^%(ALPHA_NUMERIC)s&#])(#|\uFF03)(?!%(HASHTAG_EXCLUDES)s)"
+    "(%(HASHTAG_TEXT)s)" % locals()
+)
+
+"""
+Note:
+The following regular expression is used to find hashtags in the text
+and add them to the 'Tags' section on the left panel.
+
+This pattern DOES NOT control the styling of hashtags in the text.
+To control this behaviour refer to rednotebook/files/t2t.lang
+(regexes) and rednotebook/files/rednotebook-highlight-style.xml (styles).
+
+If you make changes to this pattern, is very likely you will have to
+make changes to /rednotebook/files/t2t.lang
+"""
+HASHTAG = re.compile(HASHTAG_PATTERN, flags=re.IGNORECASE)
 
 
 def escape_tag(tag):
-    return tag.lower().replace(' ', '_')
-
-
-def convert_category_to_hashtag(category):
-    return '#{}'.format(escape_tag(category))
-
-
-def get_indexed_words(text):
-    # Strip all ASCII punctuation except for #, $, %, @ and '.
-    words = [w.strip('.|-!"&/()=?*+~_:;,<>^°`{}[]\\') for w in text.split()]
-    return [word for word in words if word]
+    return tag.lower().replace(" ", "_")
 
 
 def get_text_with_dots(text, start, end, found_text=None):
-    '''
+    """
     Find the outermost spaces and innermost newlines around
     (start, end) and add dots if needed.
-    '''
+    """
     bound1 = max(0, start - int(TEXT_RESULT_LENGTH // 2))
     bound2 = max(0, start)
     bound3 = min(end, len(text))
     bound4 = min(len(text), end + int(TEXT_RESULT_LENGTH // 2))
 
     start_values = [bound1]
-    newline = text.rfind('\n', bound1, bound2)
+    newline = text.rfind("\n", bound1, bound2)
     start_values.append(newline)
     if newline == -1:
-        start_values.append(text.find(' ', bound1, bound2))
+        start_values.append(text.find(" ", bound1, bound2))
     start = max(start_values)
 
     end_values = [bound4]
-    newline = text.find('\n', bound3, bound4)
+    newline = text.find("\n", bound3, bound4)
     if newline != -1:
         end_values.append(newline)
     else:
-        space = text.rfind(' ', bound3, bound4)
+        space = text.rfind(" ", bound3, bound4)
         if space != -1:
             end_values.append(space)
     end = min(end_values)
@@ -78,59 +85,78 @@ def get_text_with_dots(text, start, end, found_text=None):
     assert bound1 <= start <= bound2
     assert bound3 <= end <= bound4, (bound3, end, bound4)
 
-    res = ''
+    res = ""
     if start > 0:
-        res += '... '
+        res += "... "
     res += text[start:end]
     if end < len(text):
-        res += ' ...'
+        res += " ..."
 
-    res = res.replace('\n', ' ')
+    res = res.replace("\n", " ")
     if found_text:
         # Make the searched_text bold
-        res = res.replace(found_text, 'STARTBOLD%sENDBOLD' % found_text)
+        res = res.replace(found_text, f"STARTBOLD{found_text}ENDBOLD")
 
     return res
 
 
+def get_search_results_for_tags(text, tags):
+    results = []
+    add_text_to_results = False
+    for line in text.splitlines():
+        line = line.strip()
+        if any(f"#{tag}" == line.lower() for tag in tags):
+            add_text_to_results = True
+        elif any(f"#{tag}" in line.lower() for tag in tags):
+            for tag in tags:
+                # TODO: combine into one regex and really ignore case.
+                line = re.sub(rf"^#{tag}: ", "", line, re.IGNORECASE)
+                line = re.sub(rf"^#{tag}:", "", line, re.IGNORECASE)
+                line = re.sub(rf"^#{tag}", "", line, re.IGNORECASE)
+            line = line.strip()
+            results.append(line[:TEXT_RESULT_LENGTH])
+    if add_text_to_results:
+        results.append(get_text_with_dots(text, 0, TEXT_RESULT_LENGTH))
+    return results
+
+
 class Day:
     def __init__(self, month, day_number, day_content=None):
-        day_content = day_content or {'text': ''}
-        assert 'text' in day_content, day_content
+        day_content = day_content or {"text": ""}
+        assert "text" in day_content, day_content
 
         self.month = month
         self.date = datetime.date(month.year_number, month.month_number, day_number)
 
         # Turn all entries of old "Tags" categories into tags without entries.
         # Apparently, "Tags" may map to None, so explicitly convert to dict.
-        old_tags = day_content.pop('Tags', None) or {}
+        old_tags = day_content.pop("Tags", None) or {}
         for old_tag in old_tags:
             day_content[old_tag] = None
             self.month.edited = True
 
         self._content = day_content
 
-        # Remember the last edit and preview position
-        self.last_edit_pos = None
-
     def _get_content(self):
         return self._content
 
     def _set_content(self, content):
         old_text = self.text
-        new_text = content['text']
-        content['text'] = old_text
+        new_text = content["text"]
+        content["text"] = old_text
         self._content = content
         self.text = new_text
+
     content = property(_get_content, _set_content)
 
     def _get_text(self):
-        '''Return the day's text as a unicode string.'''
-        return self.content['text']
+        """Return the day's text as a unicode string."""
+        return self.content["text"]
 
     def _set_text(self, text):
-        assert 'text' in self.content
-        self.content['text'] = text
+        assert "text" in self.content
+        self.content["text"] = text
+
     text = property(_get_text, _set_text)
 
     @property
@@ -139,34 +165,7 @@ class Day:
 
     @property
     def empty(self):
-        return len(self.content) == 1 and 'text' in self.content and not self.has_text
-
-    def add_category_entry(self, category, entry):
-        if category in self.content:
-            self.content[category][entry] = None
-        else:
-            self.content[category] = {entry: None}
-
-    def merge(self, same_day):
-        assert self.date == same_day.date
-
-        # Merge texts
-        text1 = self.text.strip()
-        text2 = same_day.text.strip()
-        if text2 in text1:
-            # self.text contains the other text
-            pass
-        elif text1 in text2:
-            # The other text contains contains self.text
-            self.text = same_day.text
-            raise NotImplementedError("update search index")
-        else:
-            self.text += '\n\n' + same_day.text
-
-        # Merge categories
-        for category, entries in same_day.get_category_content_pairs().items():
-            for entry in entries:
-                self.add_category_entry(category, entry)
+        return len(self.content) == 1 and "text" in self.content and not self.has_text
 
     @property
     def hashtags(self):
@@ -181,12 +180,12 @@ class Day:
         return sorted((self.content.get(category) or {}).keys())
 
     def get_category_content_pairs(self):
-        '''
+        """
         Returns a dict of (category: content_in_category_as_list) pairs.
-        '''
+        """
         pairs = {}
         for category, content in self.content.items():
-            if category == 'text':
+            if category == "text":
                 pass
             elif content is None:
                 pairs[category] = []
@@ -198,11 +197,12 @@ class Day:
         return pairs
 
     def get_words(self, with_special_chars=False):
-        categories_text = ' '.join(
-            ' '.join([category] + content)
-            for category, content in self.get_category_content_pairs().items())
+        categories_text = " ".join(
+            " ".join([category] + content)
+            for category, content in self.get_category_content_pairs().items()
+        )
 
-        all_text = self.text + ' ' + categories_text
+        all_text = f"{self.text} {categories_text}"
         words = all_text.split()
 
         if with_special_chars:
@@ -212,24 +212,19 @@ class Day:
         words = [w.strip('.|-!"&/()=?*+~#_:;,<>^°`{}[]\\') for w in words]
         return [word for word in words if word]
 
-    def get_indexed_words(self):
-        words = []
-        for category, content in self.content.items():
-            if category == 'text':
-                pass
-            else:
-                words.append(category)
-                words.append(convert_category_to_hashtag(category))
-                if content:
-                    for entry in content.keys():
-                        words.extend(get_indexed_words(entry))
-
-        words.extend(get_indexed_words(self.text))
-        return words
-
     def get_number_of_words(self):
         return len(self.get_words(with_special_chars=True))
 
+    def replace_all(self, old, new):
+        """
+        Replaces all occurrences of old content with new.
+        Returns number of replacements.
+        """
+        self.content["text"], replaced_words = re.subn(
+            re.escape(old), repl=new, string=self.content["text"], flags=re.IGNORECASE
+        )
+        return replaced_words
+
     def search(self, text, tags):
         """
         This method is only called for days that have all given tags.
@@ -246,33 +241,36 @@ class Day:
                     # day_tags nonetheless, to escape the day_tags.
                     if escape_tag(day_tag) != tag:
                         continue
+
                     if entries:
                         results.extend(entries)
                     else:
                         add_text_to_results = True
             if add_text_to_results:
-                results.append(get_text_with_dots(self.text, 0, TEXT_RESULT_LENGTH))
+                results.extend(get_search_results_for_tags(self.text, tags))
+                # Handle right-pane tags without subentries.
+                if not results:
+                    results.append(get_text_with_dots(self.text, 0, TEXT_RESULT_LENGTH))
         elif text in str(self):
             # Date contains searched text.
             results.append(get_text_with_dots(self.text, 0, TEXT_RESULT_LENGTH))
         else:
-            text_result = self.search_in_text(text)
-            if text_result:
+            if text_result := self.search_in_text(text):
                 results.append(text_result)
             results.extend(self.search_in_categories(text))
         return str(self), results
 
     def search_in_text(self, search_text):
-        occurence = self.text.upper().find(search_text.upper())
+        occurrence = self.text.upper().find(search_text.upper())
 
         # Check if search_text is in text
-        if occurence < 0:
+        if occurrence < 0:
             return None
 
-        found_text = self.text[occurence:occurence + len(search_text)]
-        result_text = get_text_with_dots(
-            self.text, occurence, occurence + len(search_text), found_text)
-        return result_text
+        found_text = self.text[occurrence : occurrence + len(search_text)]
+        return get_text_with_dots(
+            self.text, occurrence, occurrence + len(search_text), found_text
+        )
 
     def search_in_categories(self, text):
         results = []
@@ -282,14 +280,14 @@ class Day:
                     results.extend(content)
                 else:
                     results.extend(
-                        entry for entry in content
-                        if text.upper() in entry.upper())
+                        entry for entry in content if text.upper() in entry.upper()
+                    )
             elif text.upper() in category.upper():
                 results.append(category)
         return results
 
     def __str__(self):
-        return self.date.strftime('%Y-%m-%d')
+        return self.date.strftime("%Y-%m-%d")
 
 
 class Month:
@@ -311,10 +309,11 @@ class Month:
         return self.days[day_number]
 
     def __str__(self):
-        lines = ['Month %s %s' % (self.year_number, self.month_number)]
-        for day_number, day in self.days.items():
-            lines.append('%s: %s' % (day_number, day.text))
-        return '\n'.join(lines)
+        lines = [f"Month {self.year_number} {self.month_number}"]
+        lines.extend(
+            f"{day_number}: {day.text}" for day_number, day in self.days.items()
+        )
+        return "\n".join(lines)
 
     @property
     def empty(self):
